"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Footage = exports.Layer = exports.Camera = exports.Light = exports.Mask = exports.Effect = exports.SourceRect = exports.Masks = exports.Effects = exports.MaterialOptions = exports.Text = exports.TextMoreOptions = exports.TextPathOptions = exports.SourceText = exports.TextStyle = exports.Transform = exports.PathProperty = exports.Property = exports.PropertyGroup = exports.Comp = exports.MarkerProperty = exports.MarkerKey = exports.Project = exports.Key = void 0;
// Global objects, attributes, and methods
/**
 * Keyframe objects, which can be accessed via the property method `property.key()`
 */
class Key {
    constructor(keyValue) {
        /**
         * The location of the keyframe in time
         */
        this.time = 0;
        /**
         * The index of the keyframe, e.g. The `1`st keyframe on the property. Starts from 0.
         */
        this.index = 1;
        this.value = keyValue;
    }
}
exports.Key = Key;
class Project {
    constructor() {
        /**
         * The platform-specific absolute file path, including the project file name. If the project has not been saved, it returns an empty string.
         */
        this.fullPath = "path/to/project/file";
        /**
         * The color depth of the project in bits per channel (bpc), as set in Project Settings > Color Management
         */
        this.bitsPerChannel = "8";
        /**
         * The state of the Blend Colors Using 1.0 Gamma option in Project Settings > Color Management
         */
        this.linearBlending = true;
    }
}
exports.Project = Project;
/**
 * Composition or Layer marker objects
 */
class MarkerKey {
    constructor() {
        /**
         * Duration, in seconds, of marker.
         */
        this.duration = 0;
        /**
         * Contents of Comment field in marker dialog box.
         */
        this.comment = "Marker comment";
        /**
         * Contents of Chapter field in marker dialog box.
         */
        this.chapter = "Chapter 1";
        /**
         * Contents of URL field in marker dialog box.
         */
        this.url = "URL";
        /**
         * Contents of Frame Target field in marker dialog box.
         */
        this.frameTarget = "Frame Target";
        /**
         * Setting for cue point type in marker dialog box. True for Event; false for Navigation.
         */
        this.eventCuePoint = false;
        /**
         * Contents of cue point Name field in marker dialog box.
         */
        this.cuePointName = "Cue Point Name";
        /**
         * Contents of Parameter Name and Parameter Value fields in marker dialog box.
      
        For example, if you have a parameter named “background color”, then you can use the following expression to access its value at the nearest marker:  thisComp.marker.nearestKey(time).parameters["background color"]
         */
        this.parameters = {};
        /**
         * Whether the marker represents a protected region
         */
        this.protectedRegion = false;
        /**
         * The location of the marker in time
         */
        this.time = 0;
        /**
         * The index of the marker
         */
        this.index = 1;
    }
}
exports.MarkerKey = MarkerKey;
class MarkerProperty {
    constructor() {
        /**
         * The total number of composition markers in the composition.
         */
        this.numKeys = 1;
    }
    /**
     * @returns The `Marker` object with the specified name or index
     * @param indexOrName Either the index or name (the value of the commend field) of the marker
     */
    key(indexOrName) {
        return new MarkerKey();
    }
    /**
     * @returns The marker that is nearest in time to `t`
     * @param t Time value to get the marker closest to
     */
    nearestKey(t) {
        return new MarkerKey();
    }
}
exports.MarkerProperty = MarkerProperty;
class Comp {
    constructor() {
        /**
         * The name of the composition.
         */
        this.name = "Comp Base";
        /**
         * The number of layers in the composition.
         */
        this.numLayers = 1;
        /**
         * The Camera object for the camera through which the composition is rendered at the current frame. This camera is not necessarily the camera through which you are looking in the Composition panel.
         */
        this.activeCamera = new Camera();
        /**
         * The marker property group object
         */
        this.marker = new MarkerProperty();
        /**
         * The composition width in pixels
         */
        this.width = 1920;
        /**
         * The composition height in pixels
         */
        this.height = 1080;
        /**
         * The composition duration in seconds
         */
        this.duration = 10;
        /**
         * Whether the timecode is in drop-frame format
         */
        this.ntscDropFrame = false;
        /**
         * The composition start time in seconds
         */
        this.displayStartTime = 0;
        /**
         * The duration of a frame, in seconds
         */
        this.frameDuration = 0.04;
        /**
         * The shutter-angle of the composition, in degrees
         */
        this.shutterAngle = 180;
        /**
         * The background color of the composition
         */
        this.bgColor = [1, 1, 1, 1];
        /**
         * The pixel aspect ratio of the composition
         */
        this.pixelAspect = 1;
    }
    layer(indexOrOtherLayer, relativeIndex) {
        return new Layer();
    }
}
exports.Comp = Comp;
/**
 * Groups of properties such as "Transform"
 */
class PropertyGroup {
    constructor(groupName) {
        /**
         * The name of the property group, e.g. `"Transform"`
         */
        this.name = "property group base";
        /**
         * The number of properties in the group
         */
        this.numProperties = 1;
        this.name = groupName;
    }
}
exports.PropertyGroup = PropertyGroup;
class Property {
    constructor(value, name = "Property name") {
        this.value = value;
        this.name = name;
        /**
         * The number of keyframes on the property
         */
        this.numKeys = 0;
        /**
         * The index number of the property in it's property group
         */
        this.propertyIndex = 0;
        this.velocity = this.value;
        this.speed = this.value;
    }
    /**
     * @returns The keyframe at the specified index on the property
     * @param index The index of the keyframe to return (e.g. the `1`st keyframe)
     */
    key(index) {
        return new Key(this.value);
    }
    /**
     * @returns The marker that is nearest in time to `t`
     * @param t Time value to get the marker closest to
     */
    nearestKey(time) {
        return new Key(this.value);
    }
    /**
     * @returns The group of properties (`PropertyGroup` object) relative to the property of which the expression is written
     * @param countUp The number of levels in the property hierarchy to ascend, e.g. `countUp = 1` will return the parent `PropertyGroup`.
     */
    propertyGroup(countUp) {
        return new PropertyGroup("Default property propertyGroup");
    }
    // Numeric Value methods & Properties
    // TODO: These should be undefined if the `PropertyValueType`
    // is not numeric
    /**
     * Loops a segment of time that is measured from the first keyframe on the layer forward toward the Out point of the layer. The loop plays from the In point of the layer.
     * @param type `"cycle"`: (default) Repeats the specified segment.
     * `"pingpong"`: Repeats the specified segment, alternating between forward and backward.
     * `"offset"`: Repeats the specified segment, but offsets each cycle by the difference in the value of the property at the start and end of the segment, multiplied by the number of times the segment has looped.
     * `"continue"`: Does not repeat the specified segment, but continues to animate a property based on the velocity at the first or last keyframe.
     * @param numKeyframes determines what segment is looped: The segment looped is the portion of the layer from the first keyframe to the numKeyframes+1 keyframe. The default value of 0 means that all keyframes loop
     */
    loopIn(type = "cycle", numKeyframes = 0) {
        return this.value;
    }
    /**
     * Loops a segment of time that is measured from the last keyframe on the layer back toward the In point of the layer. The loop plays until the Out point of the layer.
     * @param type `"cycle"`: (default) Repeats the specified segment.
     * `"pingpong"`: Repeats the specified segment, alternating between forward and backward.
     * `"offset"`: Repeats the specified segment, but offsets each cycle by the difference in the value of the property at the start and end of the segment, multiplied by the number of times the segment has looped.
     * `"continue"`: Does not repeat the specified segment, but continues to animate a property based on the velocity at the first or last keyframe.
     * @param numKeyframes determines what segment is looped: The segment looped is the portion of the layer from the last keyframe to the `thisProperty.numKeys - numKeyframes` keyframe. The default value of 0 means that all keyframes loop
     */
    loopOut(type = "cycle", numKeyframes = 0) {
        return this.value;
    }
    /**
     * Loops a segment of time that is measured from the first keyframe on the layer forward toward the Out point of the layer. The loop plays from the In point of the layer.
     * @param type `"cycle"`: (default) Repeats the specified segment.
     * `"pingpong"`: Repeats the specified segment, alternating between forward and backward.
     * `"offset"`: Repeats the specified segment, but offsets each cycle by the difference in the value of the property at the start and end of the segment, multiplied by the number of times the segment has looped.
     * `"continue"`: Does not repeat the specified segment, but continues to animate a property based on the velocity at the first or last keyframe.
     * @param duration The number of composition seconds in a segment to loop; the specified range is measured from the first keyframe
     */
    loopInDuration(type = "cycle", duration = 0) {
        return this.value;
    }
    /**
     * Loops a segment of time that is measured from the last keyframe on the layer back toward the In point of the layer. The loop plays until the Out point of the layer.
     * @param type `"cycle"`: (default) Repeats the specified segment.
     * `"pingpong"`: Repeats the specified segment, alternating between forward and backward.
     * `"offset"`: Repeats the specified segment, but offsets each cycle by the difference in the value of the property at the start and end of the segment, multiplied by the number of times the segment has looped.
     * `"continue"`: Does not repeat the specified segment, but continues to animate a property based on the velocity at the first or last keyframe.
     * @param duration The number of composition seconds in a segment to loop; the specified range is measured from the last keyframe backwards.
     */
    loopOutDuration(type = "cycle", duration = 0) {
        return this.value;
    }
    /**
     * @returns The temporal velocity value at the specified time. For spatial properties, such as Position, it returns the tangent vector value. The result is the same dimension as the property.
     * @param time The composition time in seconds to get the velocity at
     */
    velocityAtTime(time) {
        return this.velocity;
    }
    /**
     * @returns  A 1D, positive speed value equal to the speed at which the property is changing at the specified time. This element can be used only for spatial properties.
     * @param time The composition time in seconds to get the speed at
     */
    speedAtTime(time) {
        return this.speed;
    }
    /**
     * Returns the value for the property at the specified time
     * @param time THe composition time in seconds to get the value at
     */
    valueAtTime(time) {
        return this.value;
    }
    /**
     * Modifies the property value randomly over time.
     * @param freq The rate at which the value changes in wiggles per second
     * @param amp How much the value should change, in units of the original property value (e.g. `1` by 100% of the original value)
     * @param octaves How much detail the wiggle has, which is driven by the number of "octaves" of noise to multiply together. Higher values will have more detail
     * @param amp_mult The amount the given amplitude is multiplied by for each octave, which controls the falloff of the upper harmonics ("octaves").
     * @param time The time at which the value is sampled for use within the wiggle
     */
    wiggle(freq, amp, octaves = 1, amp_mult = 0.5, time = 0) {
        const som = freq + amp + octaves + amp_mult + time;
        return this.value;
    }
    /**
     * Samples the property value at a time which is wiggled
     * @param freq The rate at which the value changes in wiggles per second
     * @param amp How much the value should change, in units of the original property value (e.g. `1` by 100% of the original value)
     * @param octaves How much detail the wiggle has, which is driven by the number of "octaves" of noise to multiply together. Higher values will have more detail
     * @param amp_mult The amount the given amplitude is multiplied by for each octave, which controls the falloff of the upper harmonics ("octaves").
     * @param time The time at which the value is sampled for use within the wiggle
     */
    temporalWiggle(freq, amp, octaves = 1, amp_mult = 0.5, time = 0) {
        return this.value;
    }
    /**
     * Smooths the property values over time, converting large, brief deviations in the value to smaller, more evenly distributed deviations. This smoothing is accomplished by applying a box filter to the value of the property at the specified time.
     * @param width The range of time (in seconds) over which the filter is averaged.
     * @param samples The number of discrete samples evenly spaced over time; use a larger value for greater smoothness (but decreased performance). Generally, you’ll want samples to be an odd number so that the value at the current time is included in the average.
     */
    smooth(width = 0.2, samples = 5, time = 0) {
        return this.value;
    }
}
exports.Property = Property;
class PathProperty extends Property {
    /**
     * Creates a path object from a set of points and tangents.
     * @param points An array of number pair arrays representing x,y coordinates of the path points. The array length must be at least 1, and can be of any greater length.
     * @param inTangents An array containing number pair arrays representing the `[x,y]` offset coordinates of the tangent handles to the path points. Required unless no parameters are passed (i.e., `createPath()`). The array length must be the same as points, or you can pass an empty array (`[]`), which will assume the same length as points and `[0,0]` for all tangents.
     * @param outTangents See `inTangents`
     * @param isClosed Determines if the mask is closed. If true, the last point will be connected to the first point.
     */
    createPath(points = [
        [0, 0],
        [100, 0],
        [100, 100],
        [0, 100],
    ], inTangents = [], outTangents = [], is_closed = true) {
        return {};
    }
    /**
     * @returns Whether a path is closed (the last point connected to the first)
     */
    isClosed() {
        return true;
    }
    /**
     * Retrieves the points array for a path
     * @param time The time at which to sample the path
     */
    points(time = 0) {
        return [
            [0, 0],
            [0, 0],
            [0, 0],
            [0, 0],
        ];
    }
    /**
     * Retrieves the path's in tangent point array
     * @param time The time at which to sample the path
     */
    inTangents(time) {
        return [
            [0, 0],
            [0, 0],
            [0, 0],
            [0, 0],
        ];
    }
    /**
     * Retrieves the path's out tangent point array
     * @param time The time at which to sample the path
     */
    outTangents(time) {
        return [
            [0, 0],
            [0, 0],
            [0, 0],
            [0, 0],
        ];
    }
    /**
     * Get the x,y coordinates of an arbitrary point along a path.
     * @param percentage How far along the path to get the point, between 0 and 1.
     * @param time The time at which to sample the path
     */
    pointOnPath(percentage = 0.5, time = 0) {
        return [0, 0];
    }
    /**
     * Get the calculated x,y coordinates of the outgoing tangent handle for an arbitrary point along a path.
     * @param percentage How far along the path to get the point, between 0 and 1.
     * @param time The time at which to sample the path
     */
    tangentOnPath(percentage, time) {
        return [0, 0];
    }
    /**
     * Get the calculated x,y coordinates of the normal for an arbitrary point along a path.
     * @param percentage How far along the path to get the point, between 0 and 1.
     * @param time The time at which to sample the path
     */
    normalOnPath(percentage, time) {
        return [0, 0];
    }
    constructor(value = {}) {
        super(value);
    }
}
exports.PathProperty = PathProperty;
class Transform extends PropertyGroup {
    constructor() {
        super("Transform");
        /**
         * The opacity of the layer, as a percentage.
         */
        this.opacity = new Property(100, "Opacity");
        /**
         * The anchor point value of the layer in the coordinate system of the layer (layer space).
         */
        this.anchorPoint = new Property([0, 0], "Anchor Point");
        /**
         * The position value of the layer, in world space if the layer has no parent. If the layer has a parent, it returns the position value of the layer in the coordinate system of the parent layer (in the layer space of the parent layer).
         */
        this.position = new Property([0, 0], "Position");
        /**
         * The x position value of the layer, in world space if the layer has no parent. If the layer has a parent, it returns the position value of the layer in the coordinate system of the parent layer (in the layer space of the parent layer).
         */
        this.xPosition = new Property(0, "X Position");
        /**
         * The y position value of the layer, in world space if the layer has no parent. If the layer has a parent, it returns the position value of the layer in the coordinate system of the parent layer (in the layer space of the parent layer).
         */
        this.yPosition = new Property(0, "Y Position");
        /**
         * The z position value of the layer, in world space if the layer has no parent. If the layer has a parent, it returns the position value of the layer in the coordinate system of the parent layer (in the layer space of the parent layer).
         */
        this.zPosition = new Property(0, "Z Position");
        /**
         * The scale value of the layer, expressed as a percentage.
         */
        this.scale = new Property([0, 0], "Scale");
        /**
         * Returns the rotation value of the layer in degrees. For a 3D layer, it returns the z rotation value in degrees.
         */
        this.rotation = new Property(0, "Rotation");
        /**
         * Returns the 3D orientation value, in degrees, for a 3D layer.
         */
        this.orientation = new Property([0, 0, 0], "Orientation");
        /**
         * Returns the x rotation value, in degrees, for a 3D layer.
         */
        this.rotationX = new Property(0, "X Rotation");
        /**
         * Returns the y rotation value, in degrees, for a 3D layer.
         */
        this.rotationY = new Property(0, "Y Rotation");
        /**
         * Returns the z rotation value, in degrees, for a 3D layer.
         */
        this.rotationZ = new Property(0, "Z Rotation");
    }
}
exports.Transform = Transform;
class TextStyle {
    constructor() {
        /**
         * Font size of the style
         */
        this.fontSize = 0;
        /**
         * Font of the style
         */
        this.font = "Arial";
        /**
         * Whether faux bold is enabled for a style
         */
        this.isFauxBold = false;
        /**
         * Whether faux italic is enabled
         */
        this.isFauxItalic = false;
        /**
         * Whether all caps is enabled for a style
         */
        this.isAllCaps = false;
        /**
         * Whether small caps is enabled for a style
         */
        this.isSmallCaps = false;
        /**
         * Tracking value for a style
         */
        this.tracking = 0;
        /**
         * The leading value for a style
         */
        this.leading = 60;
        /**
         * Whether auto leading is enabled for a style
         */
        this.autoLeading = false;
        this.baselineDirection = "default";
        this.baselineOption = "default";
        /**
         * The baseline shift value for a style
         */
        this.baselineShift = 0;
        /**
         * Whether to apply a fill to the style
         */
        this.applyFill = true;
        /**
         * The fill color of a style
         */
        this.fillColor = [1, 1, 1];
        /**
         * Whether to apply a stroke to the style
         */
        this.applyStroke = false;
        /**
         * The stroke colour of a style
         */
        this.strokeColor = [1, 1, 1];
        /**
         * The stroke width for a style
         */
        this.strokeWidth = 0;
        this.digitSet = "default";
        this.horizontalScaling = 100;
        this.verticalScaling = 100;
        this.kerning = 0;
        this.kerningType = "metrics";
        this.isLigature = false;
        this.lineJoin = "miter";
        this.tsume = 0;
    }
    /**
     * Set the font size for a style
     * @param fontSize Font size in pixels
     * @param s Start index
     * @param n Number of characters
     */
    setFontSize(fontSize, s, n) {
        this.fontSize = fontSize;
        return this;
    }
    /**
     * Set the font for a style
     * @param font The typeface to set
     * @param s Start index
     * @param n Number of characters
     */
    setFont(font, s, n) {
        this.font = font;
        return this;
    }
    /**
     * Set the text content of a sourceText property, used when you need to return a `style` object *and* set the `value`
     * @param text The string to set
     */
    setText(text) {
        return this;
    }
    /**
     * Set the faux bold property for a style
     * @param s Start index
     * @param n Number of characters
     */
    setFauxBold(isFauxBold, s, n) {
        this.isFauxBold = isFauxBold;
        return this;
    }
    /**
     * Set the faux italic property for a style
     * @param s Start index
     * @param n Number of characters
     */
    setFauxItalic(isFauxItalic, s, n) {
        this.isFauxItalic = isFauxItalic;
        return this;
    }
    /**
     * Set the all caps property for a style
     * @param s Start index
     * @param n Number of characters
     */
    setAllCaps(isAllCaps, s, n) {
        this.isAllCaps = isAllCaps;
        return this;
    }
    /**
     * Set the small caps property for a style
     * @param s Start index
     * @param n Number of characters
     */
    setSmallCaps(isSmallCaps, s, n) {
        this.isSmallCaps = isSmallCaps;
        return this;
    }
    /**
     * Set the tracking style for a style
     * @param s Start index
     * @param n Number of characters
     */
    setTracking(tracking, s, n) {
        this.tracking = tracking;
        return this;
    }
    /**
     * Set the leading value for a style
     * @param s Start index
     * @param n Number of characters
     */
    setLeading(leading, s, n) {
        this.leading = leading;
        return this;
    }
    /**
     * Set the auto leading property for a style
     * @param s Start index
     * @param n Number of characters
     */
    setAutoLeading(autoLeading, s, n) {
        this.autoLeading = autoLeading;
        return this;
    }
    /**
     *
     * @param s Start index
     * @param n Number of characters
     */
    setBaselineDirection(baselineDirection, s, n) {
        this.baselineDirection = baselineDirection;
        return this;
    }
    /**
     *
     * @param s Start index
     * @param n Number of characters
     */
    setBaselineOption(baselineOption, s, n) {
        this.baselineOption = baselineOption;
        return this;
    }
    /**
     * Set the baseline shift value for a style
     * @param baselineShift The baseline shift to set
     * @param s Start index
     * @param n Number of characters
     */
    setBaselineShift(baselineShift, s, n) {
        this.baselineShift = baselineShift;
        return this;
    }
    /**
     * Enable or disable the fill for a style
     * @param s Start index
     * @param n Number of characters
     */
    setApplyFill(applyFill, s, n) {
        this.applyFill = applyFill;
        return this;
    }
    /**
     * Set the fill color for a style
     * @param fillColor The color to set
     * @param s Start index
     * @param n Number of characters
     */
    setFillColor(fillColor, s, n) {
        this.fillColor = fillColor;
        return this;
    }
    /**
     * Enable or disable the stroke for a style
     * @param s Start index
     * @param n Number of characters
     */
    setApplyStroke(applyStroke, s, n) {
        this.applyStroke = applyStroke;
        return this;
    }
    /**
     * Set the stroke colour for a style
     * @param strokeColor The color to set
     * @param s Start index
     * @param n Number of characters
     */
    setStrokeColor(strokeColor, s, n) {
        this.strokeColor = strokeColor;
        return this;
    }
    /**
     * Set the stroke width for a style
     * @param strokeWidth The stroke width to set
     * @param s Start index
     * @param n Number of characters
     */
    setStrokeWidth(strokeWidth, s, n) {
        this.strokeWidth = strokeWidth;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setDigitSet(digitSet, s, n) {
        this.digitSet = digitSet;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setHorizontalScaling(horizontalScaling, s, n) {
        this.horizontalScaling = horizontalScaling;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setVerticalScaling(verticalScaling, s, n) {
        this.verticalScaling = verticalScaling;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setKerning(kerning, s, n) {
        this.kerning = kerning;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setKerningType(kerningType, s, n) {
        this.kerningType = kerningType;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setLigature(ligature, s, n) {
        this.isLigature = ligature;
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setLineJoin(lineJoin, s, n) {
        this.lineJoin = lineJoin;
        return this;
    }
    /**
     * Replaces the content between `s` and `n` with the given `content`, or if `s` and `n` aren't given replaces the entire string
     * @param s Start index
     * @param n Number of characters
     */
    replaceText(content, s, n) {
        return this;
    }
    /**
     * @param s Start index
     * @param n Number of characters
     */
    setTsume(tsume, s, n) {
        this.tsume = tsume;
        return this;
    }
}
exports.TextStyle = TextStyle;
class SourceText extends Property {
    constructor(value) {
        super(value);
        this.style = new TextStyle();
    }
    /**
     * Get the style object of a text property at a character index
     * @param characterIndex Which character to get the style at
     * @param sampleTime The time to get the style at, defaulting to the current time
     */
    getStyleAt(characterIndex, sampleTime = 0) {
        return this.style;
    }
    /**
     * Used to create a new style object, rather than referencing and modifying an existing one.
     * @returns An empty style object
     */
    createStyle() {
        return new TextStyle();
    }
}
exports.SourceText = SourceText;
class TextPathOptions extends PropertyGroup {
    constructor() {
        super("Path Options");
        this.path = "Mask 1";
        this.reversePath = false;
        this.perpendicularToPath = new Property(false, "Perpendicular To Path");
        this.forceAlignment = new Property(false, "Force Alignment");
        this.firstMargin = new Property(0, "First Margin");
        this.lastMargin = new Property(0, "Last Margin");
    }
}
exports.TextPathOptions = TextPathOptions;
class TextMoreOptions extends PropertyGroup {
    constructor() {
        super("More Options");
        this.anchorPointGrouping = 1;
        this.groupingAlignment = new Property([0, 0], "Grouping Alignment");
        this.fillAndStroke = 1;
        this.interCharacterBlending = 1;
    }
}
exports.TextMoreOptions = TextMoreOptions;
class Text extends PropertyGroup {
    constructor() {
        super("Text");
        this.sourceText = new SourceText("Source text value");
        this.pathOption = new TextPathOptions();
        this.moreOption = new TextMoreOptions();
    }
}
exports.Text = Text;
class MaterialOptions extends PropertyGroup {
    constructor() {
        super("Material Options");
        this.lightTransmission = new Property(0, "Light Transmission");
        this.castShadows = new Property(false, "Cast Shadows");
        this.acceptsShadows = new Property(false, "Accept Shadows");
        this.acceptsLights = new Property(false, "Accepts Lights");
        this.ambient = new Property(100, "Ambient");
        this.diffuse = new Property(100, "Diffuse");
        this.specular = new Property(100, "Specular");
        this.shininess = new Property(100, "Shininess");
        this.metal = new Property(100, "Metal");
    }
}
exports.MaterialOptions = MaterialOptions;
class Effects extends PropertyGroup {
}
exports.Effects = Effects;
class Masks extends PropertyGroup {
}
exports.Masks = Masks;
class SourceRect {
    constructor() {
        this.top = 0;
        this.left = 0;
        this.width = 100;
        this.height = 100;
    }
}
exports.SourceRect = SourceRect;
class Effect {
    constructor() {
        /**
         * Returns true if the effect is turned on
         */
        this.active = true;
    }
    /**
     * @returns A property within an effect, e.g. `"Slider"`
     * @param nameOrIndex The name or index of the property to retrieve
     */
    param(nameOrIndex) {
        return new Property("Effect Param");
    }
}
exports.Effect = Effect;
class Mask {
    constructor() {
        /**
         * The opacity value of a mask as a percentage.
         */
        this.maskOpacity = new Property(100, "Mask Opacity");
        /**
         * The feather value of a mask, in pixels.
         */
        this.maskFeather = new Property(100, "Mask Feather");
        /**
         * The expansion value of a mask, in pixels.
         */
        this.maskExpansion = new Property(0, "Mask Expansion");
        /**
         * True if the mask is inverted or false if it is not.
         */
        this.invert = new Property(false, "Invert");
    }
}
exports.Mask = Mask;
class Light {
    constructor() {
        /**
         * The point of interest values for a light in world space.
         */
        this.pointOfInterest = new Property([0, 0, 0], "Point of Interest");
        /**
         * The intensity values of a light as a percentage.
         */
        this.intensity = new Property(100, "Intensity");
        /**
         * The color value of a light.
         */
        this.color = new Property([1, 1, 1, 1], "Color");
        /**
         * The shadow darkness value of a light as a percentage.
         */
        this.shadowDarkness = new Property(100, "Shadow Darkness");
        /**
         * The shadow diffusion value of a light, in pixels.
         */
        this.shadowDiffusion = new Property(0, "Shadow Diffusion");
        this.coneAngle = new Property(90, "Cone Angle");
        this.coneFeather = new Property(50, "Cone Feather");
    }
}
exports.Light = Light;
class Camera {
    constructor() {
        /**
         * The point of interest values of a camera in world space.
         */
        this.pointOfInterest = new Property([0, 0, 0], "Point of Interest");
        /**
         * The zoom values of a camera in pixels
         */
        this.zoom = new Property(1000, "Zoom");
        /**
         * Returns 1 if the Depth Of Field property of a camera is on, or returns 0 if the Depth Of Field property is off.
         */
        this.depthOfField = new Property(1, "Depth of Field");
        /**
         * The focus distance value of a camera, in pixels.
         */
        this.focusDistance = new Property(1000, "Focus Distance");
        /**
         * The aperture value of a camera, in pixels.
         */
        this.aperture = new Property(4, "Aperture");
        /**
         * The blur level value of a camera as a percentage.
         */
        this.blurLevel = new Property(100, "Blur Level");
        /**
         * True if the camera is the active camera for the composition at the current time: the Video switch for the camera layer is on, the current time is in the range from the In point of the camera layer to the Out point of the camera layer, and it is the first (topmost) such camera layer listed in the Timeline panel. False otherwise.
         */
        this.active = true;
    }
}
exports.Camera = Camera;
const thisComp = new Comp();
class Layer {
    constructor() {
        /**
         * The composition time, in seconds, at which the expression is being evaluated.
         */
        this.time = 0;
        /**
         * The project color depth value. For example, colorDepth returns 16 when the project color depth is 16 bits per channel.
         */
        this.colorDepth = 8;
        /**
         * The name of the layer
         */
        this.name = "Layer name";
        /**
         * The source Comp or source Footage object for the layer. Default time is adjusted to the time in the source
         */
        this.source = thisComp;
        /**
         * The width of the layer in pixels, same as `source.width`
         */
        this.width = 1920;
        /**
         * The height of the layer in pixels, same as `source.height`
         */
        this.height = 1080;
        /**
         * The index number of the layer in the composition
         */
        this.index = 0;
        /**
         * The parent Layer object of the layer, if it has one
         */
        this.parent = undefined;
        /**
         * Whether the layer has a parent layer
         */
        this.hasParent = true;
        /**
         * The in point of the layer, in seconds
         */
        this.inPoint = 0;
        /**
         * The out point of the layer in seconds
         */
        this.outPoint = 1;
        /**
         * The start time of the layer in seconds
         */
        this.startTime = 0;
        /**
         * Whether the layer has video data
         */
        this.hasVideo = true;
        /**
         * Whether the layer has audio data
         */
        this.hasAudio = true;
        /**
         * Whether the video switch is enabled, and the current time is between in the `inPoint` and `outPoint` of the layer
         */
        this.active = true;
        /**
         * Whether the video switch for the layer is enabled
         */
        this.enabled = true;
        /**
         * Whether the audio switch is enabled, and the current time is in between the `inPoint` and `outPoint` of the layer
         */
        this.audioActive = true;
        /**
         *  The value of the Audio Levels property of the layer, in decibels. This value is a 2D value; the first value represents the left audio channel, and the second value represents the right. The value is not the amplitude of the audio track of the source material. Instead, it is the value of the Audio Levels property, which may be affected by keyframes.
         */
        this.audioLevels = new Property([0, 0], "Audio Levels");
        /**
         * The value of the Time Remap property, in seconds, if Time Remap is enabled.
         */
        this.timeRemap = new Property(0, "Time Remap");
        /**
         * The marker property group object
         */
        this.marker = new MarkerProperty();
        /**
         * The transform property group object
         */
        this.transform = new Transform();
        /**
         * The text property group object
         */
        this.text = new Text();
        /**
         * The material options property group object
         */
        this.materialOption = new MaterialOptions();
    }
    /**
     * Transforms a given vector from the layer's space to the composition space
     *
     * @param vec The vector to transform
     * @param time The time to sample the vector
     * @returns The vector in the composition space
     */
    toComp(vec, time = this.time) {
        return vec;
    }
    /**
     * Transforms a given vector from the compositions space to the layer's space
     *
     * @param vec The vector to transform
     * @param time The time to sample the vector
     * @returns The vector in the layer's space
     */
    fromComp(vec, time = this.time) {
        return vec;
    }
    /**
     * Transforms a given vector from the layers space to the view-independent world space
     *
     * @param vec The vector to transform
     * @param time The time to sample the number
     * @returns The vector in world space
     */
    toWorld(vec, time = this.time) {
        return vec;
    }
    toCompVec(vec, time = this.time) {
        return vec;
    }
    fromCompVec(vec, time = this.time) {
        return vec;
    }
    toWorldVec(vec, time = this.time) {
        return vec;
    }
    fromWorldVec(vec, time = this.time) {
        return vec;
    }
    /**
     * Projects a point located in composition space to a point on the surface of the layer (zero z-value) at the location where it appears when viewed from the active camera.
     *
     * @param vec The vector to transform
     * @param time The time to sample the number
     * @returns The vector in on the layers surface space
     */
    fromCompToSurface(vec, time = this.time) {
        return vec;
    }
    /**
     * Returns the layer's source item at the given time
     * @param time The time at which to get the source
     * @returns The source item
     */
    sourceTime(time = this.time) {
        return new Footage();
    }
    /**
     * Gets the layer's size and position at a given time
     * @param time The time at which to get the layers bounds
     * @param includeExtents Whether to include areas of the layer outside the bounding box. Applies to Shape Layers and Paragraph Text.
     * @returns An object with properties for layers `top`, `left`, `width` and `height` values at the given time.
     */
    sourceRectAtTime(time = this.time, includeExtents = false) {
        return new SourceRect();
    }
    /**
     * Get the effect on a layer with a given name or index.
     * @param nameOrIndex The effect's name or index
     * @returns The first effect with the given name, or at the given index
     */
    effect(nameOrIndex) {
        return new Effect();
    }
    /**
     * Get the mask on a layer with a given name or index.
     * @param nameOrIndex The mask's name or index
     * @returns The first mask with the given name, or at the given index
     */
    mask(nameOrIndex) {
        return new Mask();
    }
    /**
     * Sample a layers color at a given point
     * @param point The center point of the sampling area, in layer space
     * @param radius Defines the sample area size, the horizontal and vertical distance from the center
     * @param postEffect Whether to sample the layer after effects and masks are applied
     * @param time The time at which to sample
     * @returns The average color of the layer in the sample area
     */
    sampleImage(point, radius = [0.5, 0.5], postEffect = true, time = this.time) {
        return [0, 0, 0, 0];
    }
    /**
     * Convert a given value in degrees to radians
     * @param degrees The value to convert
     * @returns The value radians
     */
    degreesToRadians(degrees) {
        return degrees;
    }
    /**
     * Convert a given value in radians to degrees
     * @param radians The value to convert
     * @returns The value radians
     */
    radiansToDegrees(radians) {
        return radians;
    }
    /**
     * Gets the footage object for the item with the provided name
     * @param name The file name of the footage item
     * @returns The relevant footage item
     */
    footage(name) {
        return new Footage();
    }
    /**
     * Retrieves a composition by name
     * @param name The name of the composition
     * @returns The composition with the given name
     */
    comp(name) {
        return thisComp;
    }
    /**
     * Converts a given time in seconds to an integer amount of frames
     * @param t The time to convert in seconds
     * @param fps Frames per second to calculate with, defaulting to the compositions frame rate
     * @param isDuration Whether `t` represents a duration rather than an absolute time. Durations are rounded away from zero rather than down.
     * @returns The time in frames
     */
    timeToFrames(t = this.time + thisComp.displayStartTime, fps = 1.0 / thisComp.frameDuration, isDuration = false) {
        return Math.floor(t * fps);
    }
    /**
     * Converts a number of frames to time in seconds
     * @param frames The frame count to convert
     * @param fps The frames per second use in the calculation
     * @returns The given frames as time
     */
    framesToTime(frames, fps = 1.0 / thisComp.frameDuration) {
        return frames * thisComp.frameDuration;
    }
    /**
     * Converts the given time value to a timecode string (e.g. `"00:00:00:00"`)
     * @param t The time to convert
     * @param timecodeBase The frames per second to use in the calculation
     * @param isDuration Whether `t` represents a duration rather than an absolute time. Durations are rounded away from zero rather than down.
     * @returns The time as a timecode string
     */
    timeToTimecode(t = this.time + thisComp.displayStartTime, timecodeBase = 30, isDuration = false) {
        return "00:00:00:00";
    }
    /**
     * Converts a given time value to a NTSC timecode string
     * @param t The time to convert
     * @param ntscDropFrame
     * @param isDuration Whether `t` represents a duration rather than an absolute time. Durations are rounded away from zero rather than down.
     */
    timeToNTSCTimecode(t = this.time + thisComp.displayStartTime, ntscDropFrame = false, isDuration = false) {
        return "00:00:00:00";
    }
    /**
     * Converts a given time in seconds to a string representing feet of film and frames.
     * @param t The time to convert
     * @param fps Frame rate to use for the conversion
     * @param framesPerFoot Number of frames in one foot of film
     * @param isDuration Whether `t` represents a duration rather than an absolute time. Durations are rounded away from zero rather than down.
     */
    timeToFeetAndFrames(t = this.time + thisComp.displayStartTime, fps = 1.0 / thisComp.frameDuration, framesPerFoot = 16, isDuration = false) {
        return "00:00:00:00";
    }
    /**
     * Converts a given time in seconds to the current time display format of the Project.
     * @param t The time to convert
     * @param fps Frame rate to use for the conversion
     * @param isDuration Whether `t` represents a duration rather than an absolute time. Durations are rounded away from zero rather than down.
     * @param ntscDropFrame
     */
    timeToCurrentFormat(t = this.time + thisComp.displayStartTime, fps = 1.0 / thisComp.frameDuration, isDuration = false, ntscDropFrame = thisComp.ntscDropFrame) {
        return "0000";
    }
    /**
     * Adds two vectors
     */
    add(a, b) {
        return a;
    }
    /**
     * Subtracts two vectors
     */
    sub(a, b) {
        return a;
    }
    /**
     * Multiplies a vector by a given scalar amount
     * @param vec1 The vector to multiply
     * @param amount The amount to multiply by
     */
    mul(vec1, amount) {
        return vec1;
    }
    /**
     * Divides a vector by a given scalar amount
     * @param vec1 The vector to divide
     * @param amount The amount to divide by
     */
    div(vec1, amount) {
        return vec1;
    }
    /**
     * Constrains a given number, or each element of an array, to fall within a a given range
     * @param value Array or number to constrain
     * @param limit1 Lower limit
     * @param limit2 Upper limit
     */
    clamp(value, limit1, limit2) {
        return value;
    }
    /**
     * @returns the dot (inner) product of two vectors
     */
    dot(vec1, vec2) {
        return vec1;
    }
    /**
     * @returns the cross product of two vectors
     */
    cross(vec1, vec2) {
        return vec1;
    }
    /**
     * @returns The given vector normalized so it has a length of 1
     */
    normalize(vec1) {
        return [1, 1];
    }
    /**
     * @returns The length of a given vector, or if two vectors are provided the distance between them
     */
    length(point1, point2) {
        return 1;
    }
    /**
     * Used to orient a layer towards a given point in 3D space
     * @param fromPoint The location in world space of the layer you want to orient
     * @param atPoint The point in world space you want to point the layer at
     * @returns An orientation value that can be used to orient the layer so that the z-axis points at the `atPoint`
     */
    lookAt(fromPoint, atPoint) {
        return [0, 0, 0];
    }
    /**
     * Used to modify the random seed for an expression
     * @param offset A value used to modify the random seed
     * @param timeless Whether the random seed should be consistent across time
     */
    seedRandom(offset, timeless = false) { }
    /**
     * @returns a random value either between `0` and `1`, `0` and the first argument, or the first and second argument if two are provided. If the arguments are arrays, an equal length array of random values will be returned
     * @param minValOrArray If only one argument is provided, the max value for the random number, otherwise the minimum value
     * @param maxValOrArray The maximum value to return
     */
    random(minValOrArray, maxValOrArray) {
        return minValOrArray || 0;
    }
    /**
     * @returns a random value with a gaussian distribution either between `0` and `1`, `0` and the first argument, or the first and second argument if two are provided. If the arguments are arrays, an equal length array of random values will be returned
     * @param minValOrArray If only one argument is provided, the max value for the random number, otherwise the minimum value
     * @param maxValOrArray The maximum value to return
     */
    gaussRandom(minValOrArray, maxValOrArray) {
        return minValOrArray || 0;
    }
    /**
     * Used to get a random value via Perlin noise, where inputs values that are close together will result in output values that are closer together.
     * @param valOrArray The noise input value
     * @returns A value between `-1` and `1`
     */
    noise(valOrArray) {
        return 1;
    }
    /**
     * @returns A given value, mapped from one range to another, clamped to the output range. If only 3 parameters are given, the input range is `0` to `1` and the given values are used for the output range.
     * @param t The input value to be re-mapped
     * @param tMin The inputs low floor
     * @param tMax The inputs high ceiling
     * @param value1 The output floor
     * @param value2 The output ceiling
     */
    linear(t, tMin, tMax, value1, value2) {
        return value1 || tMin;
    }
    /**
     * @returns A given value, mapped from one range to another, clamped to the output range. The mapping will ease in and out so it reaches the output range with a velocity of `0`. If only 3 parameters are given, the input range is `0` to `1` and the given values are used for the output range.
     * @param t The input value to be re-mapped
     * @param tMin The inputs low floor
     * @param tMax The inputs high ceiling
     * @param value1 The output floor
     * @param value2 The output ceiling
     */
    ease(t, tMin, tMax, value1, value2) {
        return value1 || tMin;
    }
    /**
     * @returns A given value, mapped from one range to another, clamped to the output range. The mapping will ease out with a velocity of `0`. If only 3 parameters are given, the input range is `0` to `1` and the given values are used for the output range.
     * @param t The input value to be re-mapped
     * @param tMin The inputs low floor
     * @param tMax The inputs high ceiling
     * @param value1 The output floor
     * @param value2 The output ceiling
     */
    easeIn(t, tMin, tMax, value1, value2) {
        return value1 || tMin;
    }
    /**
     * @returns A given value, mapped from one range to another, clamped to the output range. The mapping will ease into the output range  with a velocity of `0`. If only 3 parameters are given, the input range is `0` to `1` and the given values are used for the output range.
     * @param t The input value to be re-mapped
     * @param tMin The inputs low floor
     * @param tMax The inputs high ceiling
     * @param value1 The output floor
     * @param value2 The output ceiling
     */
    easeOut(t, tMin, tMax, value1, value2) {
        return value1 || tMin;
    }
    /**
     * Converts a color in RGBA space to HSLA
     * @param rgbaArray Input RGBA array of values between 0 and 1
     * @returns An array of hue, saturation, lightness and alpha values between 0 and 1
     */
    rgbToHsl(rgbaArray) {
        return [1, 1, 1, 1];
    }
    /**
     * Converts a color in HSLA space to RGBA
     * @param rgbaArray Input HSLA array of values between 0 and 1
     * @returns An array of red, green, blue and alpha values between 0 and 1
     */
    hslToRgb(hslaArray) {
        return [1, 1, 1, 1];
    }
    /**
     * Converts a color in hex triplet space to RGB, or in hex quartet space to RGBA space. For hex triplets, alpha defaults to 1.0
     * @param hex String representing an hex triplet (6 digits, no alpha channel) or quartet (8 digits, includes alpha channel) containing only numerals or characters A–F. Optional leading characters 0x, 0X, or # are ignored. Characters beyond 8 digits are ignored.
     */
    hexToRgb(hex) {
        return [1, 1, 1, 1];
    }
}
exports.Layer = Layer;
class Footage {
    constructor() {
        /**
         * The name of the footage item as shown in the project panel
         */
        this.name = "Layer Name";
        /**
         * The width of the footage item, in pixels
         */
        this.width = 500;
        /**
         * The height of the footage item, in pixels
         */
        this.height = 500;
        /**
         * The duration of the footage item, in seconds
         */
        this.duration = 10;
        /**
         * The duration of a frame in the footage item, in seconds
         */
        this.frameDuration = 0.04;
        /**
         * Whether the timecode is in NTSC drop-frame format
         */
        this.ntscDropFrame = false;
        /**
         * The pixel aspect ratio of the footage
         */
        this.pixelAspect = 1;
        /**
         * The contents of a JSON file as a string
         */
        this.sourceText = "Source Text";
        /**
         * The data of a JSON file as an array of `sourceData` objects
         */
        this.sourceData = [["source data"]];
    }
    /**
     * @returns The value of specified static or dynamic data stream in a .mgJSON file
     * @param dataPath the path in the hierarchy to the desired data stream
     */
    dataValue(dataPath) {
        return 0;
    }
    /**
     * @returns The number of samples in a specified dynamic data stream in a .mgJSON file
     * @param dataPath the path in the hierarchy to the desired dynamic data stream
     */
    dataKeyCount(dataPath) {
        return 0;
    }
    /**
     * @returns The time in seconds for the samples of a specified dynamic data stream in a .mgJSON file
     * @param dataPath The path in the hierarchy to a dynamic data stream.
     * @param t0 The start time, in seconds, of the span from which to return samples. Defaults to startTime.
     * @param t1 The end time, in seconds, of the span from which to return samples. Defaults to endTime.
     */
    dataKeyTimes(dataPath, t0, t1) {
        return [0, 0];
    }
    /**
     * @returns The values for the samples of a specified dynamic data stream in a .mgJSON file.
     * @param dataPath The path in the hierarchy to a dynamic data stream.
     * @param t0 The start time, in seconds, of the span from which to return samples. Defaults to startTime.
     * @param t1 The end time, in seconds, of the span from which to return samples. Defaults to endTime.
     */
    dataKeyValues(dataPath, t0, t1) {
        return [0, 0];
    }
}
exports.Footage = Footage;
